var moment = require('moment');

function assertDuration(test, duration, params) {
  var m = moment.duration(params);
  test.equal(duration.asMilliseconds(), m.asMilliseconds());
  test.equal(duration.asSeconds(), m.asSeconds());
  test.equal(duration.asMinutes(), m.asMinutes());
  test.equal(duration.asHours(), m.asHours());
  test.equal(duration.asDays(), m.asDays());
  test.equal(duration.asWeeks(), m.asWeeks());
  test.equal(duration.asMonths(), m.asMonths());
  test.equal(duration.asYears(), m.asYears());
}

exports.duration = {
  'parse years': function (test) {
    assertDuration(test, moment.isoDuration('P1Y'), {years: 1});
    assertDuration(test, moment.isoDuration('P2Y'), {years: 2});
    test.done();
  },

  'parse months': function (test) {
    assertDuration(test, moment.isoDuration('P1M'), {months: 1});
    assertDuration(test, moment.isoDuration('P12M'), {months: 12});
    assertDuration(test, moment.isoDuration('P36M'), {months: 36});
    test.done();
  },

  'parse weeks': function (test) {
    assertDuration(test, moment.isoDuration('P1W'), {weeks: 1});
    assertDuration(test, moment.isoDuration('P4W'), {weeks: 4});
    assertDuration(test, moment.isoDuration('P8W'), {weeks: 8});
    test.done();
  },

  'parse days': function (test) {
    assertDuration(test, moment.isoDuration('P1D'), {days: 1});
    assertDuration(test, moment.isoDuration('P31D'), {days: 31});
    assertDuration(test, moment.isoDuration('P50D'), {days: 50});
    test.done();
  },

  'parse hours': function (test) {
    assertDuration(test, moment.isoDuration('PT1H'), {hours: 1});
    assertDuration(test, moment.isoDuration('PT24H'), {hours: 24});
    assertDuration(test, moment.isoDuration('PT36H'), {hours: 36});
    test.done();
  },

  'parse minutes': function (test) {
    assertDuration(test, moment.isoDuration('PT1M'), {minutes: 1});
    assertDuration(test, moment.isoDuration('PT60M'), {minutes: 60});
    assertDuration(test, moment.isoDuration('PT90M'), {minutes: 90});
    test.done();
  },

  'parse seconds': function (test) {
    assertDuration(test, moment.isoDuration('PT1S'), {seconds: 1});
    assertDuration(test, moment.isoDuration('PT60S'), {seconds: 60});
    assertDuration(test, moment.isoDuration('PT90S'), {seconds: 90});
    test.done();
  },

  'parse milliseconds': function (test) {
    assertDuration(test, moment.isoDuration('PT1Z'), {milliseconds: 1});
    assertDuration(test, moment.isoDuration('PT1000Z'), {milliseconds: 1000});
    assertDuration(test, moment.isoDuration('PT2000Z'), {milliseconds: 2000});
    test.done();
  },

  'parse combined dates': function (test) {
    assertDuration(test, moment.isoDuration('P1Y2M'), {years: 1, months: 2});
    assertDuration(test, moment.isoDuration('P2M3D'), {months: 2, days: 3});
    assertDuration(test, moment.isoDuration('P1Y3D'), {years: 1, days: 3});
    test.done();
  },

  'parse combined times': function (test) {
    assertDuration(test, moment.isoDuration('PT1H2M'), {hours: 1, minutes: 2});
    assertDuration(test, moment.isoDuration('PT1H3S'), {hours: 1, seconds: 3});
    assertDuration(test, moment.isoDuration('PT1H4Z'), {hours: 1, milliseconds: 4});
    assertDuration(test, moment.isoDuration('PT2M3S'), {minutes: 2, seconds: 3});
    assertDuration(test, moment.isoDuration('PT3S4Z'), {seconds: 3, milliseconds: 4});
    assertDuration(test, moment.isoDuration('PT1H2M3S4Z'), {hours: 1, minutes: 2, seconds: 3, milliseconds: 4});
    test.done();
  },

  'parse combined date-time': function (test) {
    assertDuration(test, moment.isoDuration('P1Y2M3DT4H5M6S7Z'), {years: 1, months: 2, days: 3, hours: 4, minutes: 5, seconds: 6, milliseconds: 7});
    test.done();
  },

  'to iso 8601 string': function (test) {
    test.equal(moment.duration({years: 1}).toISOString(), 'P1Y');
    test.equal(moment.duration({months: 1}).toISOString(), 'P1M');
    test.equal(moment.duration({days: 1}).toISOString(), 'P1D');
    test.equal(moment.duration({hours: 1}).toISOString(), 'PT1H');
    test.equal(moment.duration({minutes: 1}).toISOString(), 'PT1M');
    test.equal(moment.duration({seconds: 1}).toISOString(), 'PT1S');
    test.equal(moment.duration({milliseconds: 1}).toISOString(), 'PT1Z');
    test.done();
  }
};